import { constantRoutes } from '@/router'
import { getRouters } from '@/api/menu'
import { getInfo } from '@/api/login'
import Layout from '@/layout/index'

const permission = {
  state: {
    routes: [],
    addRoutes: []
  },
  mutations: {
    SET_ROUTES: (state, routes) => {
      state.addRoutes = routes
      state.routes = constantRoutes.concat(routes)
    }
  },
  actions: {
    // 生成路由
    GenerateRoutes({ commit }, menuName) {
      return new Promise(resolve => {
        const menuList = menuName.menuName.split(',')
        // 向后端请求路由数据
        getInfo().then(res => {
          const newRoute = []
          // 根据点击的菜单名匹配对应的菜单
          res.data.menus.forEach(item => {
            menuList.forEach(menuItem => {
              if (item.meta.title === menuItem) {
                newRoute.push(item)
              }
            })
          })
          const accessedRoutes = filterAsyncRouter(newRoute)
          if (accessedRoutes[0]) {
            const firstChild = getFirstChild(accessedRoutes[0])
            accessedRoutes.push({ path: '*', redirect: '/404', hidden: true })
            accessedRoutes.push({ path: '/', redirect: firstChild, hidden: true })
            commit('SET_ROUTES', accessedRoutes)
          }
          const routers = { accessedRoutes: accessedRoutes, getRouters: newRoute }
          resolve(routers)
        })
      })
    }
  }
}

// 遍历后台传来的路由字符串，转换为组件对象
function filterAsyncRouter(asyncRouterMap) {
  return asyncRouterMap.filter(route => {
    if (route.component) {
      // Layout组件特殊处理
      if (route.component === 'Layout') {
        route.component = Layout
      } else {
        route.component = loadView(route.component)
      }
    }
    if (route.children != null && route.children && route.children.length) {
      route.children = filterAsyncRouter(route.children)
    }
    return true
  })
}
function getFirstChild(item) {
  if (item.children && item.children.length > 0) {
    return getFirstChild(item.children[0])
  } else {
    return item.path
  }
}

export const loadView = (view) => { // 路由懒加载
  return (resolve) => require([`@/views/${view}`], resolve)
}

export default permission
